<?php

namespace Drupal\menu_twig\Twig;

use Drupal\Core\Render\Markup;
use Drupal\Core\Url;

/**
 * Twig extension with some useful functions.
 *
 * The menu_twig_link extention replace the link extension available in
 *  menu link.
 */
class MenuTwigExtension extends \Twig_Extension {

  private $twig;

  /**
   * {@inheritdoc}
   */
  public function __construct() {
    $this->twig = \Drupal::service('twig');
  }

  /**
   * {@inheritdoc}
   */
  public function getFunctions() {
    return [
      new \Twig_SimpleFunction('menu_twig_link', [$this, 'getLink']),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return 'menu_twig';
  }

  /**
   * Gets a rendered link from a url object.
   *
   * @param string $text
   *   The link text for the anchor tag as a translated string.
   * @param \Drupal\Core\Url|string $url
   *   The URL object or string used for the link.
   * @param array|\Drupal\Core\Template\Attribute $attributes
   *   An optional array or Attribute object of link attributes.
   *
   * @return array
   *   A render array representing a link to the given URL.
   */
  public function getLink($text, $url, $attributes = []) {
    if (!$url instanceof Url) {
      $url = Url::fromUri($url);
    }
    // The twig extension should not modify the original URL object, this
    // ensures consistent rendering.
    // @see https://www.drupal.org/node/2842399
    $url = clone $url;
    if ($attributes) {
      if ($attributes instanceof Attribute) {
        $attributes = $attributes->toArray();
      }
      $url->mergeOptions(['attributes' => $attributes]);
    }
    // The text has been processed by twig already, convert it to a safe object
    // for the render system.
    if ($text instanceof \Twig_Markup) {
      $text = Markup::create($text);
    }
    $menu_html = (NULL !== $url->getOption('menu_twig')) ? $url->getOption('menu_twig') : NULL;
    if ($menu_html) {
      $menu_html = $this->twig->renderInline($menu_html);
    }
    if ($text == "<none>") {
      $build = [
        '#type' => 'markup',
        '#markup' => $menu_html,
      ];
    }
    else {
      $build = [
        '#type' => 'link',
        '#title' => $text,
        '#url' => $url,
        '#suffix' => $menu_html,
      ];
    }
    return $build;
  }

}
